@@ -1,7 +1,8 @@
 import enum
 import io
 import json
-from typing import List, Optional, Dict, Union
+
+import typing as T
 
 import requests
 
@@ -92,29 +93,31 @@ class PageType(enum.IntEnum):
 
 
 # Complex or reused type annotations
-_PermissionsType = List[Permission]
-_PathOrFileType = Union[str, io.BytesIO]
-_CookiesType = List[List[Union[str, int]]]
-_ServiceNamesToAdditionalTagsType = Dict[str, List[str]]
-_ServiceNamesToActionToTagsType = Dict[str, Dict[TagAction, List[str]]]
-_ServiceNamesToStatusesToTagsType = Dict[str, Dict[TagStatus, List[str]]]
-_VerifyAccessKeyResultType = Dict[str, Union[_PermissionsType, str]]
-_AddFileResultType = Dict[str, Union[ImportStatus, str]]
-_GetTagServicesResultType = Dict[str, List[str]]
-_UrlFilesResultType = Dict[str, Union[str, Dict[str, Union[ImportStatus, str]]]]
-_GetURLInfoResultType = Dict[str, Union[str, URLType]]
-_PagesResultType = Dict[str, Union[str, PageType, bool, List["_PagesResultType"]]]
-_FlatPagesResultType = List[Dict[str, Union[str, PageType, bool]]]
-_PageInfoResultType = Dict[
+PermissionsType = T.List[Permission]
+PathOrFileType = T.Union[str, io.BytesIO]
+CookiesType = T.List[T.List[T.Union[str, int]]]
+ServiceNamesToAdditionalTagsType = T.Dict[str, T.List[str]]
+ServiceNamesToActionToTagsType = T.Dict[str, T.Dict[TagAction, T.List[str]]]
+ServiceNamesToStatusesToTagsType = T.Dict[str, T.Dict[TagStatus, T.List[str]]]
+VerifyAccessKeyResultType = T.Dict[str, T.Union[PermissionsType, str]]
+AddFileResultType = T.Dict[str, T.Union[ImportStatus, str]]
+GetTagServicesResultType = T.Dict[str, T.List[str]]
+UrlFilesResultType = T.Dict[str, T.Union[str, T.Dict[str, T.Union[ImportStatus, str]]]]
+GetURLInfoResultType = T.Dict[str, T.Union[str, URLType]]
+PagesResultType = T.Dict[str, T.Union[str, PageType, bool, T.List["_PagesResultType"]]]
+FlatPagesResultType = T.List[T.Dict[str, T.Union[str, PageType, bool]]]
+PageInfoResultType = T.Dict[
     str,
-    Union[
+    T.Union[
         str,
         PageType,
-        Dict[str, Dict[str, List[Dict[str, Union[str, int, bool, Dict[str, Union[str, int]]]]]]],
-        Dict[str, int],
+        T.Dict[str, T.Dict[str, T.List[T.Dict[str, T.Union[str, int, bool, T.Dict[str, T.Union[str, int]]]]]]],
+        T.Dict[str, int],
     ],
 ]
-_FileMetadataResultType = List[Dict[str, Union[int, str, bool, None, List[str], _ServiceNamesToStatusesToTagsType]]]
+FileMetadataResultType = T.List[
+    T.Dict[str, T.Union[int, str, bool, None, T.List[str], ServiceNamesToStatusesToTagsType]]
+]
 
 
 class BaseClient:
@@ -147,7 +150,7 @@ class BaseClient:
     _FILE_ROUTE = "/get_files/file"
     _THUMBNAIL_ROUTE = "/get_files/thumbnail"
 
-    def __init__(self, access_key: Optional[str] = None, api_url: str = DEFAULT_API_URL,) -> None:
+    def __init__(self, access_key: T.Optional[str] = None, api_url: str = DEFAULT_API_URL,) -> None:
         """
         See https://hydrusnetwork.github.io/hydrus/help/client_api.html for more
         information.
@@ -201,7 +204,7 @@ class BaseClient:
         response = self._api_request("GET", self._API_VERSION_ROUTE)
         return response.json()["version"]
 
-    def request_new_permissions(self, name: str, permissions: _PermissionsType) -> str:
+    def request_new_permissions(self, name: str, permissions: PermissionsType) -> str:
         """
         Register a new external program with the client. This requires the 'add from api request' mini-dialog under
         services->review services to be open, otherwise it will 403.
@@ -224,13 +227,13 @@ class BaseClient:
         response = self._api_request("GET", self._SESSION_KEY_ROUTE)
         return response.json()["session_key"]
 
-    def verify_access_key(self) -> _VerifyAccessKeyResultType:
+    def verify_access_key(self) -> VerifyAccessKeyResultType:
         """Check your access key is valid."""
 
         response = self._api_request("GET", self._VERIFY_ACCESS_KEY_ROUTE)
         return response.json()
 
-    def add_file(self, path_or_file: _PathOrFileType) -> _AddFileResultType:
+    def add_file(self, path_or_file: PathOrFileType) -> AddFileResultType:
         """
         Tell the client to import a file.
 
@@ -252,7 +255,7 @@ class BaseClient:
 
         return response.json()
 
-    def clean_tags(self, tags: List[str]) -> List[str]:
+    def clean_tags(self, tags: T.List[str]) -> T.List[str]:
         """
         Ask the client about how it will see certain tags.
 
@@ -263,7 +266,7 @@ class BaseClient:
         response = self._api_request("GET", self._CLEAN_TAGS_ROUTE, params={"tags": json.dumps(tags)})
         return response.json()["tags"]
 
-    def get_tag_services(self) -> _GetTagServicesResultType:
+    def get_tag_services(self) -> GetTagServicesResultType:
         """Ask the client about its tag services."""
 
         response = self._api_request("GET", self._GET_TAG_SERVICES_ROUTE)
@@ -271,9 +274,9 @@ class BaseClient:
 
     def add_tags(
         self,
-        hashes: List[str],
-        service_to_tags: Optional[_ServiceNamesToAdditionalTagsType] = None,
-        service_to_action_to_tags: Optional[_ServiceNamesToActionToTagsType] = None,
+        hashes: T.List[str],
+        service_to_tags: T.Optional[ServiceNamesToAdditionalTagsType] = None,
+        service_to_action_to_tags: T.Optional[ServiceNamesToActionToTagsType] = None,
         add_siblings_and_parents: bool = True,
     ) -> None:
         """
@@ -297,7 +300,7 @@ class BaseClient:
 
         self._api_request("POST", self._ADD_TAGS_ROUTE, json=json_)
 
-    def get_url_files(self, url: str) -> _UrlFilesResultType:
+    def get_url_files(self, url: str) -> UrlFilesResultType:
         """
         Ask the client about an URL's files.
 
@@ -308,7 +311,7 @@ class BaseClient:
         response = self._api_request("GET", self._GET_URL_FILES_ROUTE, params={"url": url})
         return response.json()
 
-    def get_url_info(self, url: str) -> _GetURLInfoResultType:
+    def get_url_info(self, url: str) -> GetURLInfoResultType:
         """
         Ask the client for information about a URL.
 
@@ -322,12 +325,12 @@ class BaseClient:
     def add_url(
         self,
         url: str,
-        page_key: Optional[str] = None,
-        page_name: Optional[str] = None,
+        page_key: T.Optional[str] = None,
+        page_name: T.Optional[str] = None,
         show_page: bool = False,
-        service_names_to_additional_tags: Optional[_ServiceNamesToAdditionalTagsType] = None,
-        filterable_tags: List[str] = None,
-    ) -> Dict[str, str]:
+        service_names_to_additional_tags: T.Optional[ServiceNamesToAdditionalTagsType] = None,
+        filterable_tags: T.List[str] = None,
+    ) -> T.Dict[str, str]:
         """
         Tell the client to 'import' a URL. This triggers the exact same routine as drag-and-dropping a text URL onto the
         main client window.
@@ -358,7 +361,7 @@ class BaseClient:
         return response.json()
 
     def associate_url(
-        self, hashes: List[str], add: Optional[List[str]] = None, delete: Optional[List[str]] = None,
+        self, hashes: T.List[str], add: T.Optional[T.List[str]] = None, delete: T.Optional[T.List[str]] = None,
     ) -> None:
         """
         Manage which URLs the client considers to be associated with which files.
@@ -379,7 +382,7 @@ class BaseClient:
 
         self._api_request("POST", self._ASSOCIATE_URL_ROUTE, json=json_)
 
-    def get_cookies(self, domain: str) -> _CookiesType:
+    def get_cookies(self, domain: str) -> CookiesType:
         """
         Get the cookies for a particular domain.
 
@@ -389,7 +392,7 @@ class BaseClient:
         response = self._api_request("GET", self._GET_COOKIES_ROUTE, params={"domain": domain})
         return response.json()["cookies"]
 
-    def set_cookies(self, cookies: _CookiesType) -> None:
+    def set_cookies(self, cookies: CookiesType) -> None:
         """
         Set some new cookies for the client. This makes it easier to 'copy' a login from a web browser or similar to
         hydrus if hydrus's login system can't handle the site yet.
@@ -399,14 +402,14 @@ class BaseClient:
         """
         self._api_request("POST", self._SET_COOKIES_ROUTE, json={"cookies": cookies})
 
-    def get_pages(self) -> _PagesResultType:
+    def get_pages(self) -> PagesResultType:
         """
         Get the page structure of the current UI session.
         """
         response = self._api_request("GET", self._GET_PAGES_ROUTE)
         return response.json()["pages"]
 
-    def get_page_info(self, page_key: str, simple: bool = False) -> _PageInfoResultType:
+    def get_page_info(self, page_key: str, simple: bool = False) -> PageInfoResultType:
         """
         Arguments:
             page_key: hexadecimal page_key as stated in /manage_pages/get_pages
@@ -427,7 +430,7 @@ class BaseClient:
         """
         self._api_request("POST", self._FOCUS_PAGE_ROUTE, json={"page_key": page_key})
 
-    def search_files(self, tags: List[str], inbox: bool = False, archive: bool = False) -> List[int]:
+    def search_files(self, tags: T.List[str], inbox: bool = False, archive: bool = False) -> T.List[int]:
         """
         Search for the client's files.
 
@@ -448,8 +451,11 @@ class BaseClient:
         return response.json()["file_ids"]
 
     def file_metadata(
-        self, hashes: Optional[List[str]] = None, file_ids: Optional[List[int]] = None, only_identifiers: bool = False,
-    ) -> _FileMetadataResultType:
+        self,
+        hashes: T.Optional[T.List[str]] = None,
+        file_ids: T.Optional[T.List[int]] = None,
+        only_identifiers: bool = False,
+    ) -> FileMetadataResultType:
         """
         Get metadata about files in the client.
 
@@ -472,7 +478,7 @@ class BaseClient:
         response = self._api_request("GET", self._FILE_METADATA_ROUTE, params=params)
         return response.json()["metadata"]
 
-    def get_file(self, hash_: Optional[str] = None, file_id: Optional[int] = None) -> requests.Response:
+    def get_file(self, hash_: T.Optional[str] = None, file_id: T.Optional[int] = None) -> requests.Response:
         """
         Get a file.
 
@@ -483,7 +489,7 @@ class BaseClient:
         if not bool(hash_) ^ bool(file_id):
             raise RuntimeError("Pass exactly one of: hash, file_id")
 
-        params: Dict[str, Union[str, int]] = {}
+        params: T.Dict[str, T.Union[str, int]] = {}
         if hash_:
             params["hash"] = hash_
         elif file_id:
@@ -491,7 +497,7 @@ class BaseClient:
 
         return self._api_request("GET", self._FILE_ROUTE, params=params, stream=True)
 
-    def get_thumbnail(self, hash_: Optional[str] = None, file_id: Optional[int] = None) -> requests.Response:
+    def get_thumbnail(self, hash_: T.Optional[str] = None, file_id: T.Optional[int] = None) -> requests.Response:
         """
         Get a file's thumbnail.
 
@@ -502,7 +508,7 @@ class BaseClient:
         if not bool(hash_) ^ bool(file_id):
             raise RuntimeError("Pass exactly one of: hash, file_id")
 
-        params: Dict[str, Union[str, int]] = {}
+        params: T.Dict[str, T.Union[str, int]] = {}
         if hash_:
             params["hash"] = hash_
         elif file_id:
@@ -513,8 +519,8 @@ class BaseClient:
 
 class Client(BaseClient):
     def add_and_tag_files(
-        self, paths_and_files: List[_PathOrFileType], tags: List[str], services: Optional[List[str]] = None,
-    ) -> List[_AddFileResultType]:
+        self, paths_and_files: T.List[PathOrFileType], tags: T.List[str], services: T.Optional[T.List[str]] = None,
+    ) -> T.List[AddFileResultType]:
         services = ["my tags"] if services is None else services
 
         results = []
@@ -529,7 +535,7 @@ class Client(BaseClient):
         self.add_tags(list(hashes), {service: tags for service in set(services)})
         return results
 
-    def get_pages(self, flatten: bool = False) -> Union[_PagesResultType, _FlatPagesResultType]:
+    def get_pages(self, flatten: bool = False) -> T.Union[PagesResultType, FlatPagesResultType]:
         """
         Get the page structure of the current UI session.
 
